



make.one.genetree.as.Newick <- function(multree, mu, seed) {
  set.seed(seed)
  lins <- make.gsubtree(multree, rootidx.of.multree(multree), NULL, mu)
  cat("gene tree \n")
  cat("Nearly Newick\n")
  print(lins)
  tree <- nodetimeNewick.to.nodelist(lins)
  cat("Node list\n")
  print.tree.as.nodelist(tree)
  newick <- nodelist.to.Newick(tree, tree$root)
  newick <- paste(newick, ";", sep="")
  newick          
  }




make.gsubtree <- function(multree, idx, lins, mutrate) {
  node <- multree[[idx]]
  if (node$lft < 0) {
    nodelins <- make.lins.at.tip(node$clade, node$nlins)
    } else {
    lftlins <- make.gsubtree(multree, node$lft, lins, mutrate)
    rgtlins <- make.gsubtree(multree, node$rgt, lins, mutrate)
    nodelins <- c(lftlins, rgtlins)
    }
  t0 <- node$height
  if (node$anc > 0) {
    t1 <- multree[[node$anc]]$height
    } else {
    t1 <- t0 + 1e6    # huge time before root
    }
  if (node$lft < 0) {
    N0 <- node$tippop
    } else {
    N0 <- multree[[node$lft]]$rootpop + multree[[node$rgt]]$rootpop
    }
  if (node$anc > 0) { 
    N1 <- node$rootpop
    } else {
    N1 <- N0
    }
  if (node$hybpop > 0) {
    stopifnot(node$hybheight > 0)
    th <- node$hybheight
    Nh <- node$hybpop
    hyblins <- coalescent.samples.onebranch(t0, th, mutrate*N0, mutrate*Nh, nodelins)
    lins <- coalescent.samples.onebranch(th, t1, mutrate*N1, mutrate*N1, hyblins)
    } else {
    lins <- coalescent.samples.onebranch(t0, t1, mutrate*N0, mutrate*N1, nodelins)
    }
  lins
  }



collect.taxa <- function(multree) {
  collect.subtree.taxa(multree, rootidx.of.multree(multree), NULL)
  } 
  
collect.subtree.taxa <- function(multree, idx, taxa) { 
  node <- multree[[idx]]
  if (node$lft < 0) {
    taxa <- c(taxa, make.lins.at.tip(node$clade, node$nlins))
    } else {
    lfttaxa <- collect.subtree.taxa(multree, node$lft, taxa)
    rgttaxa <- collect.subtree.taxa(multree, node$rgt, taxa)
    taxa <- c(taxa, lfttaxa, rgttaxa)
    }
  taxa
  }


rootidx.of.multree <- function(multree) {
  j <- -1
  for (i in 1:length(multree)) {
    if (multree[[i]]$anc < 0) {
      stopifnot(j < 0)
      j <- i
      }
    }
  stopifnot(j > 0)
  j
  }


make.lins.at.tip <- function(clade, nlins) {
  lins <- NULL
  for (i in 1:nlins) {
    num <- toString(i)
    if (nchar(num) == 1) {
      num <- paste("0", num, sep="")
      }
    lins <- c(lins, paste(num, clade, sep=""))
    }
  lins
  }


  
  
###########################  COALESCENCES IN ONE BRANCH ############################

# The population = number of gene copies in a diploid genome (twice number of individuals).
# Time is measured back from present.
# At time s, pop is Ns, and at an earlier time t (so t > s) the
# pop is Nt.


# Returns a sample of coalescences within a branch.
# lins is a list of lineages (newick-ish strings for subtrees). At a coalescence a pair
# of such strings are deleted from list and a new one added. the updated lins is returned.

coalescent.samples.onebranch <- function(s, t, Ns, Nt, lins) {
  n <- length(lins)
  x <- s
  while (n > 1 &&  x < t) {
    x <- noncoal.lifetime.sample(s, t, Ns, Nt, n, x)
    if (x < t) {
      pair <- sample.int(n, 2)
      tm <- toString(round(x, digits=8))
      newgrp <- paste("(", lins[pair[1]], ":", tm, ",", lins[pair[2]], ":", tm, ")", sep="")
      lins <- lins[c(-pair[1], -pair[2])]
      lins <- c(lins, newgrp)
      stopifnot(n == length(lins)+1)
      n <- length(lins)
      }
    }
  lins
}



# At some time a in [s,t] there are nlin lineages.
# This function returns a sample of the lifetime of these nlin
# lineages going back in time before a coalescence happens
# and nlin canges to nlin-1.
noncoal.lifetime.sample <- function(s, t, Ns, Nt, nlin, a) {
  stopifnot(nlin > 1)
  stopifnot(s >= 0)
  stopifnot(t > s)
  stopifnot(Ns > 0)
  stopifnot(Nt > 0)
  stopifnot(a >= s)
  stopifnot(a < t)
  d <- Nt - Ns
  c <- t*Ns - s*Nt
  n <- 2/(nlin*(nlin-1))
  y <- -log(runif(1))
  if (Ns == Nt) {
    x <- a + n * (c/(t-s)) * y
    } else {
    x <- (c +d*a)*exp(n * (d/(t-s)) * y) - c
    x <- x/d
    }
  x
}


########################### lins -> tree -> Newick #####################################


# The string lins is nearly in Newick format, but it has
# absolute times measured from present, not branch lengths


# sets up arrays and calls build.tree.as.nodelist
nodetimeNewick.to.nodelist <- function(lins)
{
  n <- 0
  for (i in 1:nchar(lins)) {
    c <- substr(lins, i, i)
    if (c == ",") { n <- n+1 }
    }
  nnodes <- 2*n + 1
  node.time <- rep(0, nnodes)
  node.name <- rep("", nnodes)
  node.lft <- rep(0, nnodes)
  node.rgt <- rep(0, nnodes)
  node.anc <- rep(0, nnodes)
  tree <- list(node.time=node.time, node.name=node.name, node.lft=node.lft, node.rgt=node.rgt, node.anc=node.anc, root=0)
  tree <- build.tree.as.nodelist(lins, tree)
  tree
}


# recursively builds nodelist 'tree' from character string 'lins'
build.tree.as.nodelist <- function(lins, tree) {
  n <- idx.of.rootcomma(lins)
  if (n > 0) {
    pg <- lins.to.clade.and.height(substr(lins, 2, n-1))
    lfttime <- pg$hgt
    tree <- build.tree.as.nodelist(pg$clade, tree)
    lftroot <- tree$root

    pg <- lins.to.clade.and.height(substr(lins, n+1, nchar(lins)-1))
    rgttime <- pg$hgt
    tree <- build.tree.as.nodelist(pg$clade, tree)
    rgtroot <- tree$root

    stopifnot(lfttime == rgttime)

    nextnode <- rgtroot+1
    tree$node.time[nextnode] <- lfttime
    tree$node.lft[nextnode] <- lftroot
    tree$node.rgt[nextnode] <- rgtroot
    tree$node.anc[lftroot] <- nextnode
    tree$node.anc[rgtroot] <- nextnode
    tree <- list(node.time=tree$node.time, node.name=tree$node.name,
                 node.lft=tree$node.lft, node.rgt=tree$node.rgt,
                 node.anc=tree$node.anc, root=nextnode)
    } else {
    nextnode <- tree$root+1
    tree$node.time[nextnode] <- 0
    tree$node.name[nextnode] <- lins
    # 0 for lft,rgt,anc is correct; anc done later
    tree <- list(node.time=tree$node.time, node.name=tree$node.name,
                 node.lft=tree$node.lft, node.rgt=tree$node.rgt,
                 node.anc=tree$node.anc, root=nextnode)
    }
  tree
  }

# for build.tree.as.nodelist 
idx.of.rootcomma <- function(lins) {
  level <- 0
  n <- 0
  for (i in 1:nchar(lins)) {
    c <- substr(lins, i, i)
    if (c == "(") { level <- level + 1 }
    if (c == ")") { level <- level - 1 }
    if (level==1  &&  c == ",") { n <- i }
    }
  n
  }

# for build.tree.as.nodelist 
lins.to.clade.and.height <- function(lins) {
  n <- 0
  for (i in 1:nchar(lins)) {
    if (substr(lins, i, i) == ":") { n <- i }
    }
  stopifnot(n > 0)
  list(clade=substr(lins, 1, n-1), hgt=as.double(substr(lins, n+1, nchar(lins))))
}  


 

print.tree.as.nodelist <- function(tree) {
  cat(sprintf("%4s %8s %10s %4s %4s %4s\n", "num", "name", "time", "lft", "rgt", "anc"))
  for (i in 1:length(tree$node.time))
    {
    cat(sprintf("%4d %8s %10.6f %4d %4d %4d\n", i, tree$node.name[i], tree$node.time[i], 
                 tree$node.lft[i], tree$node.rgt[i], tree$node.anc[i]))
    }
  }



nodelist.to.Newick <- function(tree, node) {
  lftnode <- tree$node.lft[node]
  rgtnode <- tree$node.rgt[node]
  if (lftnode > 0) {
    lftnewick <- nodelist.to.Newick(tree, lftnode)
    rgtnewick <- nodelist.to.Newick(tree, rgtnode)
    lftbtime <- tree$node.time[node] - tree$node.time[lftnode]
    rgtbtime <- tree$node.time[node] - tree$node.time[rgtnode]
    newick <- paste("(", lftnewick, ":", toString(lftbtime), ",",
                         rgtnewick, ":", toString(rgtbtime), ")", sep="")
    } else {
    newick <- tree$node.name[node]
    }
  newick
  }

  
  
 
 